****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: createptsmasterlist.sas  
*
* Created (mm/dd/yyyy): 02/02/2017
* Last modified: 11/20/2017
* Version: 1.5
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This program will create the numerator dataset based on the first 4 POV datasets
*
*  Program inputs:                                                                                   
*	-The first 4 POV datasets (POV1, POV2, POV3, POV4)
* 
*  Program outputs:                                                                                                                                       
* 	-Dataset with valid numerator observations (_PtsMasterList)
* 
* PARAMETERS:
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  05/04/2017	AP		1. Added race_out and hispanic_out cohortfile parameters
*									2. Changed Priordays_Af enrollment to (priordays_af-1) - QRP-376
*									3. Changed EnrDaysAft for Type 4 to EpisodeEndDate - QRP-377
*
*	1.2		  06/26/2017	AP		Added Type 5 analysis
*
*	1.3	      10/03/2017	AP		Added option to include episodes with 0 days post blackout (QRP-319)
*
*	1.4	      11/20/2017	AP		Removed option to include episodes with 0 days post blackout (QRP-466)
*
*	1.5	      11/15/2017	AP		Bug fix to apply EnrDaysAft in Type 3 (QRP-457)
*
***************************************************************************************************;

%macro ms_createptsmasterlist();	

%put =====> MACRO CALLED: ms_createptsmasterlist v1.5;

	****************************************;
	* Type 1 or 2 or 5 cohort definition 	;
	****************************************;
	%IF %EVAL(&TYPE=1 | &TYPE=2 | &type=5) %THEN %DO;	   

        *Index Date Reference Cohort Criteria;
        data _PtsMasterList;
		format IndexDt EpisodeEndDt ENR_START ENR_END date9.;
        merge _POV1(in=pov1 rename=Adate=IndexDt)
              _POV2(in=pov2 rename=Adate=IndexDt drop=InGap)
              _POV3(in=pov3 rename=Adate=IndexDt keep=patid Adate)
              _POV4(in=pov4 rename=EpisodeStartDt=IndexDt)
              _IOT(in=iot rename=EpisodeStartDt=IndexDt);
        by Patid IndexDt; * Adate represents potential Index Date;		
        
        if pov1 and not pov2 and pov3 and pov4;

        *Index Date must overlap greater monitoring period;
		*infant birth date must overlap query period;
        if &startfollowup. <= birth_date <= &enddate.;

		OrigEpisEndDt = EpisodeEndDt;

		*Adjust EpisodeEndDt if EpisodeEndDt > enddate;
		%if &censor_qryend = 1 %then %do;
			if EpisodeEndDt > &enddate. then EpisodeEndDt = &enddate.;
		%end;

		/*Adjust EpisodeEndDt if EpisodeEndDt > DeathDt*/
		%if %eval(&type. = 2 | &type. = 5) %then %do; 			
			if EnrEpisodeCensoredAtDeath = 1 and EpisodeEndDt > DeathDt then EpisodeEnddt = deathdt;
		%end;

        *Trucate episode at IOT;
		*Only adjust for IOT when IOT prior to EpisodeEndDate. IOT after EpisodeEndDt can 
		 occur if Epiosde shortened to occur on EndDate;
        if iot and TrunkDate and trunkdate <= EpisodeEndDt then EpisodeEndDt=TrunkDate;

    	*ExpExtPer may have made the episode to go beyond elig.;
		*TURN OFF ELIGIBLITY CRITERIA FOR MOM-BABY LINKAGE;
/*    	if EpisodeEndDt > enr_end then EpisodeEndDt = enr_end;*/

        if blackoutper =. then blackoutper=0;

        *Minimum look back enrollment duration;
/*        if  Enr_Start <= IndexDt-max(0,&enrdays.,Washper,T2FupWashPer,-1*&PRIORDAYS_bf.,&MinCovFrom.,&L1HDPSFROM.,&MinMFUfrom.) then MinEnrolMet=1;*/
/**/
/*		if  Enr_End >= IndexDt+max(0,&PRIORDAYS_af.,&MaxCovTo.,&MaxMFUTo.,&L1HDPSTO.,&EnrDaysAft.) then MinEnrolMet2=1;*/

		MinEnrolMet = 1; MinEnrolMet2 = 1;

        *Minimum Episode Duration;
        if EpisodeEndDt-IndexDt + 1 >= minepisdur then MinEpiDurMet=1;  

		*Maximum Episode Duration;
        if not missing(maxepisdur) and EpisodeEndDt-IndexDt + 1 > maxepisdur then EpisodeEndDt = IndexDt + maxepisdur -1;  

        *Patient must be enrolled throughout the at-risk period when ITT;
        if minepisdur > 0 then do;
            if Enr_Start <= indexDt  and Indexdt + minepisdur - 1 <= Enr_end then FUEnrolMet=1;
        end;
        else FUEnrolMet=1;

		*remove episodes that do not have at least one day at risk post blackout (at least one day to observe an event);
       	if blackoutper and EpisodeEndDt - (IndexDt + blackoutper) < 0 then delete;

		*If ADS files are to be created, patients must have at least one post blackout day at risk (at least 
		one day to observe and event) in the look of their index date;
		if "&Analysis."="ps" or "&Analysis."="ads" then do;
			if blackoutper and IndexDtLookEndDt - (IndexDt + blackoutper) < 0 then delete;
		end;

        if  MinEnrolMet and MinEnrolMet2 and FUEnrolMet and MinEpiDurMet;

    	*Defensive coding;
    	if IndexDt > EpisodeEndDt then delete;                

        keep PatID IndexDt EpisodeEndDt ENR_START ENR_END 
             mindaysupp T2FupWashPer blackoutper T&type.CohortDef IndexLook washper RxSup RxAmt NumDispensing
			 %if %eval(&type.=2 | &type.=5) %then %do;
	 			OrigEpisEndDt trunkdate EnrEpisodeCensoredAtDeath maxepisdur Deathdt enr_end_equal_death
			 %end;;
        run;

		*Add w.a.r.n.i.n.g. if MinCovFrom is bigger in absolute terms than priordays;
    	%isdata(dataset=_PtsMasterList);
        %IF %EVAL(&NOBS.>=1) %THEN %DO;
			data _null_;
			set _PtsMasterList;
			MINFROM=max(0,&enrdays.,Washper,T2FupWashPer,-1*&PRIORDAYS_bf.); 
			MAXTO=max(0,&PRIORDAYS_af., &enrdaysaft.);
	        call symputx("MINFROM",MINFROM);
	        call symputx("MAXTO",MAXTO);
			run;

			%IF %EVAL(&MinCovFrom. > &MINFROM.) %THEN %DO;
				%put WARNING: many elig patients may be lost due to lookback days used for covariates (MinCovFrom);
			%END;
			%IF %EVAL(&MaxCovTo. > &MAXTO.) %THEN %DO;
				%put WARNING: many elig patients may be lost due to lookforward days used for covariates (MaxCovTo);
			%END;
			%put &MinCovFrom.  &MINFROM. &MAXTO.;
		%END;*if dataset is populated;		
		
		%if %eval(&type=2) %then %do;
			*Summarize utilization that are overlapping selected episodes;
	        %ms_shaveoutside(reffile=_PtsMasterList,
	                          refstart=IndexDt,
	                          refend=EpisodeEndDt,
	                          KeepPartBf=Y,
	                          ToShaveFile=_GroupIndex,
	                          ToShaveStart=Adate,
	                          ToShaveEnd=ExpireDt,
	                          outfile=_GroupUtilFile);

	        *Summarize RxSup during episode (only to apply mindaysupp criterion);
	        *The final summarization will be done later when episode selection is finalized;
	        proc means data=_GroupUtilFile nway noprint;
	        var RxSup RxAmt;
	        class PatId IndexDt;
	        output out=_GroupUtilFile(keep=PatId IndexDt TotRxSup) sum(RxSup)=TotRxSup;
	        run;

	        data _PtsMasterList;
	        merge _PtsMasterList(in=a)
	              _GroupUtilFile;
	        by PatId IndexDt;
	        if a;

	        if TotRxSup >= mindaysupp then MinDaySupMet=1;
	        if MinDaySupMet;

	    	drop TotRxSup washper;
	        run;
		%end;
    %END;

	****************************************;
	* Type 3 cohort definition		 		;
	****************************************;
 	%IF %EVAL(&TYPE.=3) %THEN %DO;

		data _null_;
		call symputx("ExpPeriodEndDt",INTNX('Month', &T3ENDOFUPDATE.-max(&T3RiskTo.,&T3CtrlTo.,0, &enrdaysaft., &priordays_af., &maxcovto., &maxMFUto.),0, 'begin')-1);  *for inclusion in metadatafile; 		
		run;

		data _null_;
		call symputx("MaxExpPeriodEndDt",Max(&MaxExpPeriodEndDt., &ExpPeriodEndDt.));
		run;

        *Manage All Dates;
        data _POV1;
        set _POV1;
        by PatId;
		format T3CtrlFromDt T3CtrlToDt T3RiskFromDt T3RiskToDt MinWindowDt MaxWindowDt date9.;

        *Calulate windows start and end dates for reuse;        
        T3CtrlFromDt=Adate+T3CtrlFrom;
        T3CtrlToDt=Adate+T3CtrlTo;
        T3RiskFromDt=Adate+T3RiskFrom;
        T3RiskToDt=Adate+T3RiskTo;

        *Calculate length of lookback period for HOI INCIDENCE ASSESSMENT;            
        MinWindowDt=min(ADate, T3CtrlFromDt,T3RiskFromDt);
        MaxWindowDt=max(ADate, T3CtrlToDt,T3RiskToDt);
        LookBackLength=MaxWindowDt-MinWindowDt+1;
        
		RequiredFUPPer=max(T3RiskTo,T3CtrlTo,0, &enrdaysaft., &priordays_af., &maxcovto.,&maxMFUto.);
		call symputx("ExpPeriodEndDt",INTNX('Month', &T3ENDOFUPDATE.-RequiredFUPPer,0, 'begin')-1);  *for inclusion in metadatafile; 

        *Calculate min/max elig requirement from Adate perspective and using LookBackLength;
        *Note: Exclusion elig requirement is already accounted for in POV3;
        format MinEligtDt MaxEligtDt MinEligtHOIDt MaxEligtEnrAftDt date9.;


		MinEligtDt=Adate-max(0, &enrdays., Washper, -1*&PRIORDAYS_bf.,&MinCovFrom., &MinMFUFROM.);       
		MinEligtHOIDt=min(MinEligtDt, MinWindowDt - LookBackLength);
        MaxEligtDt=max(0,MaxWindowDt,Adate);

		MaxEligtEnrAftDt=.;
		if &enrdaysaft. > 0 then MaxEligtEnrAftDt=Max(MaxEligtDt, Adate + &enrdaysaft., Adate + &PRIORDAYS_af., Adate + &MaxCovTo.,Adate + &MaxMFUTo.);
		if T3FupWashPer = . then LookBackLength=999999;
	
		LookBackLength=max(LookBackLength,T3FupWashPer);
        run;

        *IndexDt Reference Cohort Criteria (Exposure Event Criteria);
        data _PtsMasterList;
		format IndexDt ENR_START ENR_END date9.;
        merge _POV1(in=pov1 rename=Adate=IndexDt)
              _POV2(in=pov2 rename=Adate=IndexDt drop=InGap)
              _POV3(in=pov3 rename=Adate=IndexDt keep=patid Adate)
              _AgeGroups(keep=PatId IndexDt age birth_date sex race hispanic AgeGroup);
        by PatId IndexDt;
		
        if pov1 and not pov2 and pov3;

        *Minimum look back enrollment duration;
        if  Enr_Start <= MinEligtDt then MinEnrolMet=1;
		if  Enr_Start <= MinEligtHOIDt then MinEnrolHOIMet=1;
        if  Enr_End >= MaxEligtEnrAftDt then MinEnrolMet2=1;

        if  MinEnrolMet and MinEnrolMet2;        

		keep PatID IndexDt ENR_START ENR_END EnrEpisodeCensoredAtDeath MinEnrolHOIMet MinEnrolMet2 MaxEligtEnrAftDt
             MinEligtDt MaxEligtDt MinWindowDt MaxWindowDt LookBackLength age birth_date sex race hispanic AgeGroup  
             T3FupWashPer T3CohortDef T3CtrlFromDt T3CtrlToDt T3RiskFromDt T3RiskToDt MinEligtHOIDt zip: state hhs_reg cb_reg;
        run;

    %END;
	
	****************************************;
	* Type 4 cohort definition		 		;
	****************************************;
	%IF %EVAL(&TYPE.=4) %THEN %DO;	

        *IndexDt Reference Cohort Criteria;
		*Output data for enroll table;
        data _PtsMasterList 
			 _Deliveries(drop=adate birth_date has:  priority: race hispanic sex indexdt expiredt episode
							  washper t4fupwashper ConcWashPer MinEnrolMet);
		format IndexDt EpisodeEndDt ENR_START ENR_END date9.;
        merge _POV1(in=pov1 rename=Adate=IndexDt)
              _POV2(in=pov2 rename=Adate=IndexDt drop=InGap)
              _POV3(in=pov3 rename=Adate=IndexDt keep=patid Adate)
              _POV4(in=pov4 rename=EpisodeStartDt=IndexDt);              
        by Patid IndexDt; * Adate represents potential Index Date;        

		if first.PatId then DelNum=1;
		else DelNum=DelNum+1;
		retain DelNum;

		if pov1 and not pov2 and pov3 and pov4;

		* Apply the period dates criterion to episode end date (unlike type 1, 2 and 3 it is not applied to IndexDt);
		if &startfollowup. <= EpisodeEndDt <= &enddate.;
          	
        *Minimum look back enrollment duration;
        if  Enr_Start <= EpisodeEndDt -max(0,&enrdays.,Washper,T4FupWashPer) then MinEnrolMet=1; 
		if  Enr_End >= EpisodeEndDt+max(0,&EnrDaysAft.) then MinEnrolMet2=1;  

    	*Defensive coding;
    	if IndexDt > EpisodeEndDt then delete;
           
		label Code="PregDurCode"; 
		rename code=PregDurCode;
		
		*Create eligdays for enrolment tables;
		eligdays=adate-enr_start;

		drop Duration episodegap episodegap2 expextper expextper2 gap ittdays ittdays2 
			 LEnrStart LRunOutDate RXAMT RXSUP WashPer2  t4CohortDef2 t4cohortdef t4fupwashper2;		

		if  MinEnrolMet and MinEnrolMet2 then output _PtsMasterList;
		output _Deliveries;
        run;

		proc sql noprint undo_policy=none;
        create table _PtsMasterList as
        select mstr.*,
               age.AgeGroup 
        from _PtsMasterList as mstr,
             _AgeGroups as age
        where mstr.PatId=age.PatId and mstr.EpisodeEndDt=age.IndexDt
        order by Patid,IndexDt;
        quit;

		proc sql noprint undo_policy=none;
        create table _Deliveries as
        select del.*,
               age.AgeGroup 
        from _Deliveries as del,
             _AgeGroups as age
        where del.PatId=age.PatId and del.EpisodeEndDt=age.IndexDt
        order by Patid,IndexDt;
        quit;

		%if %eval(&NUM_MPN.>0) %then %do;

			data _POV1_Sec;
			format MOIName $10.;
			set 
			%do n=1 %to &NUM_MPN.;			
				_POV1&n.(in=file&n.)			
			%end;;
			%do n=1 %to &NUM_MPN.;	
				if file&n. then MOIName="MP&n.";	
			%end;
			run;

			data _POV2_Sec;
			format MOIName $10.;
			set 
			%do n=1 %to &NUM_MPN.;			
				_POV2&n.(in=file&n.)			
			%end;;
			%do n=1 %to &NUM_MPN.;	
				if file&n. then MOIName="MP&n.";	
			%end;
			run;	

			data _POV4_Sec;
			format MOIName $10.;
			set 
			%do n=1 %to &NUM_MPN.;			
				_POV4&n.(in=file&n.)			
			%end;;
			%do n=1 %to &NUM_MPN.;	
				if file&n. then MOIName="MP&n.";	
			%end;
			run;
			
			proc sql noprint;
			create table _PtsMasterList2 as
			select pov1.*,
				   pov4.Episode,
				   pov4.EpisodeStartDt as IndexDt,
				   pov4.EpisodeEndDt
			from _POV1_Sec as pov1,
				 _POV4_Sec as pov4 
			where pov1.PatId=pov4.PatId and
				  pov1.MOIName=pov4.MOIName and 
				  pov1.ADate=pov4.EpisodeStartDt			  
			order by PatId, MOIName, ADate;
			quit;

			proc sort data=_POV2_Sec;
			by PatId MOIName ADate;
			run;

			data _PtsMasterList2;
			merge _PtsMasterList2 (in=a)
			      _POV2_Sec(in=pov2 drop=InGap);
			by PatId MOIName ADate;  * Adate represents potential Index Date;
			if a and not pov2;
			drop ConcWashPer DaysUntreated episodegap: expextper: ExpireDt ittdays ittdays2 t4cohortdef t4CohortDef2
			t4fupwashper t4fupwashper2 WashPer WashPer2 ADate RxSup RxAmt CodeSupply;
			run;
		%end;
    %END;

%put NOTE: ********END OF MACRO: ms_createptsmasterlist v1.5********;

%mend ms_createptsmasterlist;
